@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: amacro.c,v 1.7 2003/05/31 13:16:09 spetm Exp $
+ * $Id: amacro.c,v 1.8 2004/04/04 19:53:44 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ parse_aperture_macro(gerb_file_t *fd)
 	 * First element describes which primitive element to use
 	 */
 	if (primitive == 0) {
-	    primitive = gerb_fgetint(fd);
+	    primitive = gerb_fgetint(fd, NULL);
 	}
 
 	switch (gerb_fgetc(fd)) {
@@ -108,7 +108,7 @@ parse_aperture_macro(gerb_file_t *fd)
 	    ip = ip->next;
 	    ip->opcode = PPUSH;
 	    amacro->nuf_push++;
-	    ip->data.ival = gerb_fgetint(fd);
+	    ip->data.ival = gerb_fgetint(fd, NULL);
 	    comma = 0;
 	    break;
 	case '*':
