@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: draw.c,v 1.33 2003/03/16 10:44:15 spetm Exp $
+ * $Id: draw.c,v 1.33.8.2 2004/12/28 00:39:41 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -144,9 +144,10 @@ image2pixmap(GdkPixmap **pixmap, struct gerb_image *image,
 {
     GdkGC *gc = gdk_gc_new(*pixmap);
     GdkGC *pgc = gdk_gc_new(*pixmap);
+    GdkGCValues gc_values;
     struct gerb_net *net;
     gint x1, y1, x2, y2;
-    int p1, p2;
+    int p1, p2, p3;
     int cir_width = 0, cir_height = 0;
     int cp_x = 0, cp_y = 0;
     GdkPoint *points = NULL;
@@ -216,11 +217,13 @@ image2pixmap(GdkPixmap **pixmap, struct gerb_image *image,
 
 	/*
 	 * Set GdkFunction depending on if this (gerber) layer is inverted
+	 * and allow for the photoplot being negative.
 	 */
-	if (net->layer_polarity == CLEAR)
-	    gdk_gc_set_function(gc, GDK_CLEAR);
+	gdk_gc_set_function(gc, GDK_COPY);
+	if ((net->layer_polarity == CLEAR) != (polarity == NEGATIVE))
+	    gdk_gc_set_foreground(gc, &opaque);
 	else
-	    gdk_gc_set_function(gc, GDK_COPY);
+	    gdk_gc_set_foreground(gc, &transparent);
 
 	/*
 	 * Polygon Area Fill routines
@@ -322,10 +325,31 @@ image2pixmap(GdkPixmap **pixmap, struct gerb_image *image,
 	case FLASH :
 	    p1 = (int)round(image->aperture[net->aperture]->parameter[0] * unit_scale);
 	    p2 = (int)round(image->aperture[net->aperture]->parameter[1] * unit_scale);
+	    p3 = (int)round(image->aperture[net->aperture]->parameter[2] * unit_scale);
 	    
 	    switch (image->aperture[net->aperture]->type) {
 	    case CIRCLE :
 		gerbv_draw_circle(*pixmap, gc, TRUE, x2, y2, p1);
+		/*
+		 * If circle has an inner diameter we must remove
+		 * that part of the circle to make a hole in it.
+		 * We should actually support square holes too,
+		 * but due to laziness I don't.
+		 */
+		if (p2) {
+		    if (p3) GERB_COMPILE_WARNING("Should be a square hole in this aperture.\n");
+		    gdk_gc_get_values(gc, &gc_values);
+		    if (gc_values.foreground.pixel == opaque.pixel) {
+			gdk_gc_set_foreground(gc, &transparent);
+			gerbv_draw_circle(*pixmap, gc, TRUE, x2, y2, p2);
+			gdk_gc_set_foreground(gc, &opaque);
+		    } else {
+			gdk_gc_set_foreground(gc, &opaque);
+			gerbv_draw_circle(*pixmap, gc, TRUE, x2, y2, p2);
+			gdk_gc_set_foreground(gc, &transparent);
+		    }
+		}
+
 		break;
 	    case RECTANGLE:
 		gerbv_draw_rectangle(*pixmap, gc, TRUE, x2, y2, p1, p2);
