@@ -2,9 +2,9 @@
  * gEDA - GNU Electronic Design Automation
  * This file is a part of gerbv.
  *
- *   Copyright (C) 2000-2001 Stefan Petersen (spe@stacken.kth.se)
+ *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: draw.h,v 1.3 2001/12/07 17:02:40 spetm Exp $
+ * $Id: draw.h,v 1.6 2002/08/01 22:02:06 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +27,15 @@
 #include <gdk/gdk.h>
 #include "gerber.h"
 
+/* Default mouse cursor. Perhaps redefine this to a variable later? */
+#define GERBV_DEF_CURSOR	NULL
+
 /*
- * Convert a gerber image to a GTK pixmap to be displayed
+ * Convert a gerber image to a GDK clip mask to be used when creating pixmap
  */
 int 
 image2pixmap(GdkPixmap **pixmap, struct gerb_image *image, 
 	     int scale, double trans_x, double trans_y,
-	     enum polarity_t polarity, 
-	     GdkColor *fg_color, GdkColor *bg_color, GdkColor *err_color);
+	     enum polarity_t polarity);
 
 #endif /* DRAW_H */
