@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerb_file.h,v 1.5 2002/10/18 21:04:53 spetm Exp $
+ * $Id: gerb_file.h,v 1.7 2004/04/04 19:53:45 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,19 +27,29 @@
 #include <stdio.h>
 
 typedef struct file {
-    FILE *fd;
-    int   fileno;
-    char *data;
-    int   datalen;
-    int   ptr;
+    FILE *fd;     /* File descriptor */
+    int   fileno; /* The integer version of fd */
+    char *data;   /* Pointer to data mmaped in. May not be changed, use ptr */
+    int   datalen;/* Length of mmaped data ie file length */
+    int   ptr;    /* Index in data where we are reading */
 } gerb_file_t;
 
+
 gerb_file_t *gerb_fopen(char *filename);
 int gerb_fgetc(gerb_file_t *fd);
-int gerb_fgetint(gerb_file_t *fd);
+int gerb_fgetint(gerb_file_t *fd, int *len); /* If len != NULL, returns number
+						of chars parsed in len */
 double gerb_fgetdouble(gerb_file_t *fd);
 char *gerb_fgetstring(gerb_file_t *fd, char term);
 void gerb_ungetc(gerb_file_t *fd);
 void gerb_fclose(gerb_file_t *fd);
 
+/*
+ * Search for files in directories pointed out by paths, a NULL terminated
+ * list of directories to search. If a string in paths starts with a $, then
+ * characters to / (or string end if no /) is interpreted as a environment
+ * variable.
+ */
+char *gerb_find_file(char *filename, char **paths);
+
 #endif /* GERB_FILE_H */
