@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerber.c,v 1.67 2004/04/04 19:53:45 spetm Exp $
+ * $Id: gerber.c,v 1.70 2004/06/27 17:47:00 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,6 +96,7 @@ parse_gerb(gerb_file_t *fd)
      */
     memset((void *)state, 0, sizeof(gerb_state_t));
     state->layer_polarity = DARK;
+
     /* 
      * "Inches are assumed if units are not specified"
      * rs274xrevd_e.pdf, p. 39
@@ -135,7 +136,7 @@ parse_gerb(gerb_file_t *fd)
 	    break;
 	case 'X':
 	    coord = gerb_fgetint(fd, &len);
-	    if (image->format->omit_zeros == TRAILING) {
+	    if (image->format && image->format->omit_zeros == TRAILING) {
 
 		switch ((image->format->x_int + image->format->x_dec) - len) {
 		case 5:
@@ -161,7 +162,7 @@ parse_gerb(gerb_file_t *fd)
 	    break;
 	case 'Y':
 	    coord = gerb_fgetint(fd, &len);
-	    if (image->format->omit_zeros == TRAILING) {
+	    if (image->format && image->format->omit_zeros == TRAILING) {
 
 		switch ((image->format->y_int + image->format->y_dec) - len) {
 		case 5:
@@ -258,10 +259,14 @@ parse_gerb(gerb_file_t *fd)
 	     * Count number of points in Polygon Area 
 	     */
 	    if (state->in_parea_fill && state->parea_start_node) {
-		if (state->aperture_state == OFF) {
-		    /*
-		     * aperture state off, end current polygon and start a new one 
-		     */
+		/* 
+		 * "...all lines drawn with D01 are considered edges of the
+		 * polygon. D02 closes and fills the polygon."
+		 * p.49 rs274xrevd_e.pdf
+		 * D02 -> state->aperture_state == OFF
+		 */
+		if (state->aperture_state == OFF &&
+		    state->interpolation != PAREA_START) {
 		    curr_net->interpolation = PAREA_END;
 		    curr_net->next = (gerb_net_t *)malloc(sizeof(gerb_net_t));
 		    curr_net = curr_net->next;
@@ -301,9 +306,7 @@ parse_gerb(gerb_file_t *fd)
 	     * gerbers don't reset the interpolation (EagleCad again).
 	     */
 	    if ((state->interpolation == PAREA_START) ||
-		(state->interpolation == PAREA_END) ||
-		(state->interpolation == MQ_START) ||
-		(state->interpolation == MQ_END))
+		(state->interpolation == PAREA_END))
 		state->interpolation = state->prev_interpolation;
 
 	    /*
@@ -329,12 +332,24 @@ parse_gerb(gerb_file_t *fd)
 		aperture_size = 0.0;
 
 	    /*
-	     * Find min and max of image with compensation for mm.
+	     * For next round we save the current position as
+	     * the previous position
 	     */
+	    state->prev_x = state->curr_x;
+	    state->prev_y = state->curr_y;
+
+	    /*
+	     * If we have an aperture defined at the moment we find 
+	     * min and max of image with compensation for mm.
+	     */
+	    if ((curr_net->aperture == 0) && !state->in_parea_fill) 
+		break;
+
 	    if (curr_net->unit == MM)
 		scale = 25.4;
 	    else 
 		scale = 1.0;
+
 	    if (image->info->min_x > curr_net->stop_x)
 		image->info->min_x = (curr_net->stop_x - aperture_size) / scale;
 	    if (image->info->min_y > curr_net->stop_y)
@@ -344,9 +359,6 @@ parse_gerb(gerb_file_t *fd)
 	    if (image->info->max_y < curr_net->stop_y)
 		image->info->max_y = (curr_net->stop_y + aperture_size) / scale;
 	    
-	    state->prev_x = state->curr_x;
-	    state->prev_y = state->curr_y;
-	    
 	    break;
 	case 10 :   /* White space */
 	case 13 :
@@ -422,13 +434,9 @@ parse_G_code(gerb_file_t *fd, gerb_state_t *state, gerb_format_t *format)
 	state->unit = MM;
 	break;
     case 74: /* Disable 360 circular interpolation */
-	state->prev_interpolation = state->interpolation;
-	state->interpolation = MQ_END;
 	state->mq_on = 0;
 	break;
     case 75: /* Enable 360 circular interpolation */
-	state->prev_interpolation = state->interpolation;
-	state->interpolation = MQ_START;
 	state->mq_on = 1;
 	break;
     case 90: /* Specify absolut format */
