@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerbv.c,v 1.140 2004/05/19 20:01:17 spetm Exp $
+ * $Id: gerbv.c,v 1.142.6.3 2005/02/08 09:09:36 motyl Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -567,6 +567,9 @@ color_selection_popup(GtkWidget *widget, gpointer data)
 static void 
 invert_color(GtkWidget *widget, gpointer data)
 {
+    if (!screen.file[screen.curr_index])
+	return;
+
     if (screen.file[screen.curr_index]->inverted)
 	screen.file[screen.curr_index]->inverted = 0;
     else
@@ -592,7 +595,7 @@ cb_ok_load_file(GtkWidget *widget, GtkFileSelection *fs)
 	filename = dirname(filename);
 	if (screen.path)
 	    free(screen.path);
-	screen.path = (char *)malloc(strlen(filename) + 1);
+	screen.path = (char *)malloc(strlen(filename) + 2);
 	strcpy(screen.path, filename);
 	screen.path = strncat(screen.path, "/", 1);
 	
@@ -689,7 +692,7 @@ cb_ok_export_png(GtkWidget *widget, GtkFileSelection *fs)
     filename = dirname(filename);
     if (screen.path)
 	free(screen.path);
-    screen.path = (char *)malloc(strlen(filename) + 1);
+    screen.path = (char *)malloc(strlen(filename) + 2);
     strcpy(screen.path, filename);
     screen.path = strncat(screen.path, "/", 1);
    
@@ -748,7 +751,7 @@ cb_ok_project(GtkWidget *widget, gpointer data)
 	 */
 	if (screen.path)
 	    free(screen.path);
-	screen.path = (char *)malloc(strlen(filename) + 1);
+	screen.path = (char *)malloc(strlen(filename) + 2);
 	strcpy(screen.path, filename);
 	dirname(screen.path);
 	screen.path = strncat(screen.path, "/", 1);
@@ -839,7 +842,7 @@ cb_ok_project(GtkWidget *widget, gpointer data)
     filename = dirname(filename);
     if (screen.path)
 	free(screen.path);
-    screen.path = (char *)malloc(strlen(filename) + 1);
+    screen.path = (char *)malloc(strlen(filename) + 2);
     strcpy(screen.path, filename);
     screen.path = strncat(screen.path, "/", 1);
 
@@ -1627,6 +1630,10 @@ open_image(char *filename, int idx, int reload)
 	return -1;
     }
 
+    /* Used to debug parser */
+    if (screen.dump_parsed_image)
+	gerb_image_dump(parsed_image);
+
     /*
      * If reload, just exchange the image. Else we have to allocate
      * a new memory before we define anything more.
@@ -1644,6 +1651,7 @@ open_image(char *filename, int idx, int reload)
     /*
      * Store filename for eventual reload
      * XXX Really should check retval from malloc!!! And use strncpy
+     * TOM: not really. filename is not changing between strlen and strcpy
      */
     screen.file[idx]->name = (char *)malloc(strlen(filename) + 1);
     strcpy(screen.file[idx]->name, filename);
@@ -2421,33 +2429,6 @@ lookup_widget (GtkWidget * widget, const gchar * widget_name)
 } /* lookup_widget */
 
 
-#ifdef HAVE_GETOPT_LONG
-int longopt_val = 0;
-int longopt_idx = 0;
-static struct option longopts[] = {
-    /* name              has_arg            flag  val */
-    {"version",          no_argument,       NULL,    'V'},
-    {"batch",            required_argument, NULL,    'b'},
-    {"log",              required_argument, NULL,    'l'},
-    {"tools",            required_argument, NULL,    't'},
-    {"geometry",         required_argument, &longopt_val, 1},
-    {"project",          required_argument, &longopt_val, 'p'},
-    /* GDK/GDK debug flags to be "let through" */
-    {"gtk-module",       required_argument, &longopt_val, 2},
-    {"g-fatal-warnings", no_argument,       &longopt_val, 2},
-    {"gtk-debug",        required_argument, &longopt_val, 2},
-    {"gtk-no-debug",     required_argument, &longopt_val, 2},
-    {"gdk-debug",        required_argument, &longopt_val, 2},
-    {"gdk-no-debug",     required_argument, &longopt_val, 2},
-    {"display",          required_argument, &longopt_val, 2},
-    {"sync",             no_argument,       &longopt_val, 2},
-    {"no-xshm",          no_argument,       &longopt_val, 2},
-    {"name",             required_argument, &longopt_val, 2},
-    {"class",            required_argument, &longopt_val, 2},
-    {0, 0, 0, 0},
-};
-#endif /* HAVE_GETOPT_LONG*/
-
 void
 rename_main_window(char *filename, GtkWidget *main_win)
 {
@@ -2468,6 +2449,36 @@ rename_main_window(char *filename, GtkWidget *main_win)
 				 
 }
 
+#ifdef HAVE_GETOPT_LONG
+int longopt_val = 0;
+int longopt_idx = 0;
+const struct option longopts[] = {
+    /* name              has_arg            flag  val */
+    {"version",          no_argument,       NULL,    'V'},
+    {"help",             no_argument,       NULL,    'h'},
+    {"batch",            required_argument, NULL,    'b'},
+    {"log",              required_argument, NULL,    'l'},
+    {"tools",            required_argument, NULL,    't'},
+    {"project",          required_argument, NULL,    'p'},
+    {"dump",             no_argument,       NULL,    'd'},
+    {"geometry",         required_argument, &longopt_val, 1},
+    /* GDK/GDK debug flags to be "let through" */
+    {"gtk-module",       required_argument, &longopt_val, 2},
+    {"g-fatal-warnings", no_argument,       &longopt_val, 2},
+    {"gtk-debug",        required_argument, &longopt_val, 2},
+    {"gtk-no-debug",     required_argument, &longopt_val, 2},
+    {"gdk-debug",        required_argument, &longopt_val, 2},
+    {"gdk-no-debug",     required_argument, &longopt_val, 2},
+    {"display",          required_argument, &longopt_val, 2},
+    {"sync",             no_argument,       &longopt_val, 2},
+    {"no-xshm",          no_argument,       &longopt_val, 2},
+    {"name",             required_argument, &longopt_val, 2},
+    {"class",            required_argument, &longopt_val, 2},
+    {0, 0, 0, 0},
+};
+#endif /* HAVE_GETOPT_LONG*/
+const char *opt_options = "Vhl:t:p:d";
+
 int
 main(int argc, char *argv[])
 {
@@ -2493,10 +2504,10 @@ main(int argc, char *argv[])
     setup_init();
 	
 #ifdef HAVE_GETOPT_LONG
-    while ((read_opt = getopt_long(argc, argv, "Vl:p:", 
+    while ((read_opt = getopt_long(argc, argv, opt_options, 
 				   longopts, &longopt_idx)) != -1) {
 #else
-    while ((read_opt = getopt(argc, argv, "Vl:p:")) != -1) {
+    while ((read_opt = getopt(argc, argv, opt_options)) != -1) {
 #endif /* HAVE_GETOPT_LONG */
 	switch (read_opt) {
 #ifdef HAVE_GETOPT_LONG
@@ -2521,9 +2532,6 @@ main(int argc, char *argv[])
 		    break;
 		req_y = (int)strtol(rest, &rest, 10);
 		break;
-	    case 'p': /* project */
-		project_filename = optarg;
-		break;
 	    default:
 		break;
 	    }
@@ -2561,15 +2569,32 @@ main(int argc, char *argv[])
 		exit(1);
             }
 	    break;
+	case 'd':
+	    screen.dump_parsed_image = 1;
+	    break;
 	case '?':
-
+	case 'h':
+#ifdef HAVE_GETOPT_LONG
 	    fprintf(stderr, "Usage : %s [FLAGS] <gerber file(s)>\n", argv[0]);
 	    fprintf(stderr, "where FLAGS could be any of\n");
 	    fprintf(stderr, "  --version|-V : Prints version of gerbv\n");
+	    fprintf(stderr, "  --help|-h : Prints this help message\n");
 	    fprintf(stderr, "  --log=<logfile>|-l <logfile> : Send error messages to <logfile>\n");
-	    fprintf(stderr, "  --tools=<ToolFile.drl> : Read Excellon tools from this file.\n");
+	    fprintf(stderr, "  --project=<prjfile>|-p <prjfile> : Load project file <prjfile>\n");
+	    fprintf(stderr, "  --tools=<toolfile>|-t <toolfile> : Read Excellon tools from file <toolfile>\n");
 	    exit(1);
 	    break;
+#else
+	    fprintf(stderr, "Usage : %s [FLAGS] <gerber file(s)>\n", argv[0]);
+	    fprintf(stderr, "where FLAGS could be any of\n");
+	    fprintf(stderr, "  -V : Prints version of gerbv\n");
+	    fprintf(stderr, "  -h : Prints this help message\n");
+	    fprintf(stderr, "  -l <logfile> : Send error messages to <logfile>\n");
+	    fprintf(stderr, "  -p <prjfile> : Load project file <prjfile>\n");
+	    fprintf(stderr, "  -t <toolfile> : Read Excellon tools from file <toolfile>\n");
+	    exit(1);
+	    break;
+#endif /* HAVE_GETOPT_LONG */
 	default :
 	    printf("Not handled option [%d=%c]\n", read_opt, read_opt);
 	}
@@ -2696,7 +2721,7 @@ main(int argc, char *argv[])
 	     */
 	    if (screen.path)
 		free(screen.path);
-	    screen.path = (char *)malloc(strlen(project_filename) + 1);
+	    screen.path = (char *)malloc(strlen(project_filename) + 2);
 	    strcpy(screen.path, project_filename);
 	    dirname(screen.path);
 	    screen.path = strncat(screen.path, "/", 1);
