@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: setup.h,v 1.1 2003/02/24 20:35:20 spetm Exp $
+ * $Id: setup.h,v 1.2 2003/12/14 10:51:36 spetm Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,16 @@
 #ifndef SETUP_H
 #define SETUP_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define DEF_DISTFONTNAME "-*-helvetica-bold-r-normal--*-120-*-*-*-*-iso8859-1"
+#ifdef USE_GTK2
+#define DEF_STATUSFONTNAME "Courier Extra-Condensed 11"
+#else
 #define DEF_STATUSFONTNAME "-*-fixed-*-*-normal--*-100-*-*-*-*-iso8859-1"
+#endif
 
 typedef struct {
     char *status_fontname;
